#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: Feb 11 2008

## convert a gff format to a wiggle format

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $


use constant { WIN=>0, CPG=>1, MET=>2, COV=>3, SUM=>4, ID=>5 };

use Getopt::Long qw(:config no_ignore_case);

my ($helpFlag, $inFile, $probeFile, $outFile, $verbose, $quiet, $window, $step);
my $bin = 10000;

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"input=s"		=> \$inFile,		## input file
	"output=s"		=> \$outFile,		## output file
	"probe=s"		=> \$probeFile,
	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"window=i"		=> \$window,
	"step=i"			=> \$step,
) || die "\n";

checkOptions();

my (%probes, $id, %refs, $out);
load($probeFile) if $probeFile;
process($inFile);

#-------------------------------------------------------------------------------

sub load
{
	my ($fileName) = @_;

	print "Loading ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);

	#chr3RHet	51206	51242	1
	while (<$in>)
	{
		next if /^#/ || /^\s*$/ || /^chrom/;
		my @a = split(/[\t\r\n]/);
		my ($wn, $st) = $window ? ($window, $step) : ($a[2]-$a[1], $a[2]);

		for (my $i = $a[1]; $i < $a[2]; $i += $st)
		{
			push(@{$probes{$a[0]}{int($i/$bin)}}, [$i, $i+$wn-1, "$a[3]:$a[4]"]);
		}
	}

	close($in);
}

sub process
{
	my ($fileName) = @_;

	print "Processing ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);
	$out = openOutput($outFile);
	print $out "#Ref\tStart\tEnd\tCpGs\tMethylated\tCoverage\tMethyPercent\tAvgMethyPercent\n";

	#chr3RHet	51206	51242	1
	while (<$in>)
	{
		next if /^#/ || /^\s*$/;
		my @a = split(/[\t\r\n]/);

		onRef() if ($id && $id ne $a[0]);

		$id = $a[0];
		my ($s, $e);

		if ($probeFile)
		{
			foreach my $b (int($a[1]/$bin)..int($a[2]/$bin))
			{
				next if !exists $probes{$a[0]}{$b};
				foreach my $f (@{$probes{$a[0]}{$b}})
				{
					add($f->[0], $f->[1], $a[3], $a[4], $f->[2]) if $f->[0] <= $a[1] && $a[1] <= $f->[1];
				}
			}
		}
		else
		{
			$s = ($a[1]+$step-$window) / $step; $s = 0 if $s < 0;
			#$e = ($a[2]-1            ) / $step; # looks like a bug
			$e = ($a[2]-2            ) / $step;

			foreach my $p (int($s)..int($e))
			{
				add($p*$step+1, $p*$step+$window, $a[3], $a[4]);
			}
		}
	}

		onRef() if $id;
	close($in) if defined $fileName;
}

sub add
{
#	my ($start, $end, $pct, $cov, $id) = @_;
	my ($start, $end, $cov, $pct, $id) = @_;
	$refs{$start}[WIN] = $end;
	$refs{$start}[CPG] ++;
	$refs{$start}[MET] += int($cov*$pct+0.5);
	$refs{$start}[COV] += $cov;
	$refs{$start}[SUM] += $pct;
	$refs{$start}[ID ] = $id;
}

sub onRef
{
	foreach my $p (sort {$a<=>$b} keys %refs)
	{
		print $out join("\t", $id, $p, @{$refs{$p}}[WIN,CPG,MET,COV]);
		printf $out "\t%.3f\t%.3f", $refs{$p}[MET]/$refs{$p}[COV], $refs{$p}[SUM]/$refs{$p}[CPG];
		print $out "\t$refs{$p}[ID]" if $refs{$p}[ID];
		print $out "\n";
	}

	%refs = ();
}

#-------------------------------------------------------------------------------

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "zcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| gzip -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	$inFile  = shift(@ARGV) if !defined $inFile  && @ARGV > 0;
	$outFile = shift(@ARGV) if !defined $outFile && @ARGV > 0;
	die "The output file name is the same as the input file name\n" if defined $inFile && defined $outFile && $inFile eq $outFile;

	if ($helpFlag || (!defined $window && !$probeFile) || (defined $window && !$step))
	{
		die("Arguments: -s step -s window_size -p probe_file [[-i] in_file] [[-o] out_file] [-v] [-q]\n"
		  );
	}
}
